import os
import platform
import array as arr

class Col(): # \u001b[38;5;${ID}m
    BLACK   = "\u001b[30m"
    RED     = "\u001b[31m"
    GREEN   = "\u001b[32m"
    YELLOW  = "\u001b[33m"
    BLUE    = "\u001b[34m"
    MAGENTA = "\u001b[35m"
    CYAN    = "\u001b[36m"
    WHITE   = "\u001b[37m"

    BG_BLACK   = "\u001b[40m"
    BG_RED     = "\u001b[41m"
    BG_GREEN   = "\u001b[42m"
    BG_YELLOW  = "\u001b[43m"
    BG_BLUE    = "\u001b[44m"
    BG_MAGENTA = "\u001b[45m"
    BG_CYAN    = "\u001b[46m"
    BG_WHITE   = "\u001b[47m"

    def bgcol(num): 
        return "\u001b[" + "48;5;" + str(num) + "m"

    def fgcol(num): 
        return "\u001b[" + "38;5;" + str(num) + "m"
 

class Display():
    """
    Text display area class
    """
    
    def __init__(self, title="", rows=50, cols=100):
        """
        Initializing the Display class

        Parameters
        ----------
        title : str
            Used to set the title using ANSI escape codes - by default it is empty
        rows : int
            Used to set the terminal's row size using ANSI escape codes - by default it is 50
        cols : int
            Used to set the terminal's column size using ANSI escape codes - by default it is 100
        """

        # Variables
        self.area = [] # Stores the contents of the next frame
        self.last = [] # Stores the contents of the last frame

        # CMD preperation 
        if platform.system() == "Windows":
            os.system("color")

        # CMD title
        print("\033]2;{}\007".format(title))

        # CMD size
        print("\x1b[8;{};{}t".format(rows, cols))

        # CMD clear
        if platform.system() == "Windows":
            os.system("cls")
        else:
            os.system("clear")
            

    def insert(self, text, x, y, escape="", bg="", fg=""):
        """
        Inserts a character into the Display's area 
        while also dynamically increasing the display area based on the coordinates given

        Parameters
        ----------
        text : str
            The character that will be inserted into position x, y
        x : int
            The x coordinate of the character that will be replaced
        y: int
            The y coordinate of the character that will be replaced
        escape : str
            An ANSI escape sequence, set to None as a blank string
        bg : int
            ANSI 256-colour code
        fg : int
            ANSI 256-colour code
        """

        # Handling type
        text = str(text)

        for i in range(len(text)):

            # Checking for y axis increase
            if y >= len(self.area):
                self.area.extend( [[]]*(y+1 - len(self.area)) )

            # Checking for x axis increase
            if x+i >= len(self.area[y]):
                self.area[y].extend( [" "]*(x+i+1 - len(self.area[y])) )

            # Formatting and inserting text
            insert_text = escape + Col.bgcol(bg) + Col.fgcol(fg) + str(text[i]) + "\u001b[0m"  # "\u001b[0m" is an ASNI reset code
            self.area[y][x+i] = u"{}".format(insert_text)


    def move(self, target_x, target_y, destination_x, destination_y):
        """
        Moves the target character to the destination
        It replaces the target character's old position with the one from the previous frame

        Parameters
        ----------
        target_x : int
            The x coordinate of the target character
        target_y : int
            The y coordinate of the target character
        destination_x : int
            The x coordinate of the destination to send the target character
        destination_y : int
            The y coordinate of the destination to send the target character
        """

        self.insert(self.area[target_y][target_x], destination_x, destination_y)

        try:
            self.area[target_y][target_x] = self.last[target_y][target_x]
        except:
            self.area[target_y][target_x] = " "

    def clear(self):
        """
        Clears the current area of all elements
        """

        self.area = []
        self.last = []

    def clear_line(self, y):
        """
        Clears a vertical by replacing every character with spaces

        y : int
            The y coordinate of the affected line
        """

        for i in range(0, len(self.area[y])):
            self.area[y][i] = " "

    def refresh(self):
        """
        Sets the cursor position to the start of the terminal 
        then writing over the old contents with the contents of the next frame.
        """

        # Setting cursor position
        print("\033[0;0H")

        # Displaying the new frame
        for line in self.area:
            print("".join(str(char) for char in line))

        # Storing the current frame as the last frame
        self.last = self.area

if __name__ == "__main__":
    app = Display(title="Colour program")

    for col in range(257): 
       app.insert(str(col), 0, col, fg=col)
       app.insert(str(col), 5, col, bg=col)
       
    app.refresh()